@@ -30,11 +30,13 @@ Jclient::Jclient (const char *jname, const char *jserv, int chan, int corr) :
     _fsamp (0),
     _fsize (0),
     _active (false),
+    _kmdsp (0),
     _chan (chan),
     _corr (corr)
 {
     init_jack (jname, jserv);   
     initialise ();
+    _active = true;
 }
 
 
@@ -76,7 +78,7 @@ void Jclient::init_jack (const char *jname, const char *jserv)
         _jack_ipports [i] = jack_port_register (_jack_client, t, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
     }
 
-    _active = true;
+    _kmdsp = new Kmeterdsp [_chan];
 }
 
 
@@ -84,6 +86,7 @@ void Jclient::close_jack ()
 {
     jack_deactivate (_jack_client);
     jack_client_close (_jack_client);
+    delete[] _kmdsp;
 }
 
 
@@ -107,6 +110,7 @@ int Jclient::jack_static_process (jack_nframes_t nframes, void *arg)
 
 int Jclient::jack_process (jack_nframes_t nframes)
 {
+    int   i;
     float *p0, *p1;
 
     if (! _active) return 0;
@@ -118,12 +122,10 @@ int Jclient::jack_process (jack_nframes_t nframes)
         _kmdsp [0].process (p0, nframes);
         _kmdsp [1].process (p1, nframes);
 	if (_corr) _scdsp.process (p0, p1, nframes);
-	if (_chan > 2)
+	for (i = 2; i < _chan; i++)
 	{
-	     p0 = (float *) jack_port_get_buffer (_jack_ipports [2], nframes);
-	     p1 = (float *) jack_port_get_buffer (_jack_ipports [3], nframes);
-            _kmdsp [2].process (p0, nframes);
-            _kmdsp [3].process (p1, nframes);
+	     p0 = (float *) jack_port_get_buffer (_jack_ipports [i], nframes);
+            _kmdsp [i].process (p0, nframes);
 	}
     }
     return 0;
