@@ -1,6 +1,6 @@
 // -----------------------------------------------------------------------------
 //
-//  Copyright (C) 2008-2018 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2008-2021 Fons Adriaensen <fons@linuxaudio.org>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@ public:
     Jclient (const char *jname, const char *jserv, int chan, int corr);
     ~Jclient (void);
 
+    int           chan (void) const { return _chan; }
     const char   *jname (void) const { return _jname; }
     unsigned int  fsamp (void) const { return _fsamp; }
     Kmeterdsp    *kmdsp (int i) const { return (Kmeterdsp *)_kmdsp + i; }
@@ -53,12 +54,12 @@ private:
     void initialise (void);
 
     jack_client_t  *_jack_client;
-    jack_port_t    *_jack_ipports [4];
+    jack_port_t    *_jack_ipports [MAXCHAN];
     const char     *_jname;
     unsigned int    _fsamp;
     unsigned int    _fsize;
     bool            _active;
-    Kmeterdsp       _kmdsp [4];
+    Kmeterdsp      *_kmdsp;
     Stcorrdsp       _scdsp;
     int             _chan;
     int             _corr;
