@@ -1,6 +1,6 @@
 // -----------------------------------------------------------------------------
 //
-//  Copyright (C) 2008-2018 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2008-2020 Fons Adriaensen <fons@linuxaudio.org>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -38,6 +38,12 @@ XImage *Kmeter::_k14_meterV0 = 0;
 XImage *Kmeter::_k14_meterV1 = 0;
 XImage *Kmeter::_k14_scaleH  = 0;
 XImage *Kmeter::_k14_scaleV  = 0;
+XImage *Kmeter::_k12_meterH0 = 0;
+XImage *Kmeter::_k12_meterH1 = 0;
+XImage *Kmeter::_k12_meterV0 = 0;
+XImage *Kmeter::_k12_meterV1 = 0;
+XImage *Kmeter::_k12_scaleH  = 0;
+XImage *Kmeter::_k12_scaleV  = 0;
 
 
 Kmeter::Kmeter (X_window *parent, int xpos, int ypos, int geom, int kval) :
@@ -58,10 +64,17 @@ Kmeter::Kmeter (X_window *parent, int xpos, int ypos, int geom, int kval) :
     case K20:
         _imag0 = (geom == HOR) ? _k20_meterH0 : _k20_meterV0; 
         _imag1 = (geom == HOR) ? _k20_meterH1 : _k20_meterV1; 
+        _scale = (geom == HOR) ? _k20_scaleH : _k20_scaleV; 
 	break;
     case K14:
         _imag0 = (geom == HOR) ? _k14_meterH0 : _k14_meterV0; 
         _imag1 = (geom == HOR) ? _k14_meterH1 : _k14_meterV1; 
+        _scale = (geom == HOR) ? _k14_scaleH  : _k14_scaleV; 
+	break;
+    case K12:
+        _imag0 = (geom == HOR) ? _k12_meterH0 : _k12_meterV0; 
+        _imag1 = (geom == HOR) ? _k12_meterH1 : _k12_meterV1; 
+        _scale = (geom == HOR) ? _k12_scaleH  : _k12_scaleV; 
 	break;
     default:
 	return;
@@ -99,8 +112,24 @@ void Kmeter::update (float r, float p)
 {
     int x, y, kr, kp, dp;
 
-    kr = (_kval == K20) ? mapk20 (r) : mapk14 (r);
-    kp = (_kval == K20) ? mapk20 (p) : mapk14 (p);
+    switch (_kval)
+    {
+    case K20:
+        kr = (int)(mapk20 (r) + 0.5f);
+        kp = (int)(mapk20 (p) + 0.5f);
+	break;
+    case K14:
+        kr = (int)(mapk14 (r) + 0.5f);
+        kp = (int)(mapk14 (p) + 0.5f);
+	break;
+    case K12:
+        kr = (int)(mapk12 (r) + 0.5f);
+        kp = (int)(mapk12 (p) + 0.5f);
+	break;
+    default:
+	return;
+    }	
+	
     dp = kp - kr;
     if (dp >  3) dp = 3;
     if (kp < 24) dp = 0;
@@ -158,56 +187,73 @@ void Kmeter::update (float r, float p)
 }
 
 
-int Kmeter::mapk20 (float v)
+float Kmeter::mapk20 (float v)
 {
-    if (v < 1e-3f) return (int)(24e3f * v);
-    v = log10f (v) + 3;
-    if (v < 2.0f) return (int)(24.3f + v * (100 + v * 16));
+    if (v < 1e-3f) return 24e3f * v;
+    v = log10f (v) + 3.0f;
+    if (v < 2.0f) return 24 + v * (100 + v * 16);
     if (v > 3.0f) v = 3.0f;
-    return (int)(v * 160 - 31.7f);
+    return v * 160 - 32;
 }
 
 
-int Kmeter::mapk14 (float v)
+float Kmeter::mapk14 (float v)
 {
-    if (v < 2e-3f) return (int)(12e3f * v);
+    if (v < 2e-3f) return 10e3f * v;
     v = log10f (v) + 2.7f;
-    if (v < 2.0f) return (int)(20.3f + v * (80 + v * 32));
+    if (v < 2.0f) return 20 + v * (80 + v * 32);
     if (v > 2.7f) v = 2.7f;
-    return (int)(v * 200 - 91.7f);
+    return v * 200 - 92;
+}
+
+ 
+float Kmeter::mapk12 (float v)
+{
+    if (v < 2.5e-3f) return 8e3f * v;
+    v = log10f (v) + 2.6f;
+    if (v < 2.0f) return 20 + v * (90 + v * 32);
+    if (v > 2.7f) v = 2.7f;
+    return v * 200 - 72;
 }
 
  
 void Kmeter::load_images (X_display *disp, const char *path)
 {
-    char s [1024];
-
     if (_k20_meterH0) return;
     _peakcol = disp->whitepixel ();
 
-    snprintf (s, 1024, "%s/k20-meterH0.png", path);
-    _k20_meterH0 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k20-meterH1.png", path);
-    _k20_meterH1 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k20-meterV0.png", path);
-    _k20_meterV0 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k20-meterV1.png", path);
-    _k20_meterV1 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k20-scaleH.png", path);
-    _k20_scaleH = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k20-scaleV.png", path);
-    _k20_scaleV = png2img (s, disp, 0);
+    _k20_meterH0 = read_png ("k20-meterH0", path, disp);
+    _k20_meterH1 = read_png ("k20-meterH1", path, disp);
+    _k20_meterV0 = read_png ("k20-meterV0", path, disp);
+    _k20_meterV1 = read_png ("k20-meterV1", path, disp);
+    _k20_scaleH = read_png ("k20-scaleH", path, disp);
+    _k20_scaleV = read_png ("k20-scaleV", path, disp);
 
-    snprintf (s, 1024, "%s/k14-meterH0.png", path);
-    _k14_meterH0 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k14-meterH1.png", path);
-    _k14_meterH1 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k14-meterV0.png", path);
-    _k14_meterV0 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k14-meterV1.png", path);
-    _k14_meterV1 = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k14-scaleH.png", path);
-    _k14_scaleH = png2img (s, disp, 0);
-    snprintf (s, 1024, "%s/k14-scaleV.png", path);
-    _k14_scaleV = png2img (s, disp, 0);
+    _k14_meterH0 = read_png ("k14-meterH0", path, disp);
+    _k14_meterH1 = read_png ("k14-meterH1", path, disp);
+    _k14_meterV0 = read_png ("k14-meterV0", path, disp);
+    _k14_meterV1 = read_png ("k14-meterV1", path, disp);
+    _k14_scaleH = read_png ("k14-scaleH", path, disp);
+    _k14_scaleV = read_png ("k14-scaleV", path, disp);
+
+    _k12_meterH0 = read_png ("k12-meterH0", path, disp);
+    _k12_meterH1 = read_png ("k12-meterH1", path, disp);
+    _k12_meterV0 = read_png ("k12-meterV0", path, disp);
+    _k12_meterV1 = read_png ("k12-meterV1", path, disp);
+    _k12_scaleH = read_png ("k12-scaleH", path, disp);
+    _k12_scaleV = read_png ("k12-scaleV", path, disp);
 }
+
+
+XImage* Kmeter::read_png (const char *file, const char *path, X_display *disp)
+{
+    char     s [1024];
+    XImage   *I;
+    
+    snprintf (s, 1024, "%s/%s.png", path, file);
+    I = png2img (s, disp, 0);
+    if (I) return I;
+    fprintf (stderr, "Can't load '%s'.\n", s);
+    exit (1);
+}
+    
